<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class SmartpackAPI
 */
class SmartpackAPI
{
    const VERSION = '1.0.0';

    // Singleton w/o Dependency Injection
    protected static $instance;

    // Which environemnt to connect to
    protected $appId;
    protected $accessToken;
    protected $endpoint;

    protected $quotes = [];
    protected $drop_point_translation;

    public function __construct()
    {
        $config = json_decode(base64_decode(get_option('smartpack_access_str')));
        if ($config->endpoint) {
            $this->appId = $config->appId;
            $this->accessToken = $config->accessToken;
            $this->endpoint = 'https://' . $config->endpoint . '/api/v1';
        } else {
            throw new Exception('Configuration smartpack_access_str not set');
        }
    }

    /**
     * @return SmartpackAPI
     * @throws Exception
     */
    public static function instance()
    {
        if (!self::$instance) {
            self::$instance = new SmartpackAPI();
        }
        return self::$instance;
    }

    /**
     * Fetches all shipping rates for
     * the order channel
     *
     * @return array
     */
    public function getShippingRates()
    {
        try {
            $data = [
                'data' => [
                    'type' => 'rate_quotes'
                ]
            ];


            $res = $this->request($this->endpoint . '/shopcheckout/list', 'post', (object)[]);
            $body = $res['data'];
        } catch (Exception $e) {
            error_log('Get shipping rates failed: ' . $e->getMessage());
            $body = [];
        }

        return $body;
    }

    /**
     * Perform a rate quote lookup
     * and return any available rates
     * for the shipping route
     *
     * @return array An array of rates
     */
    public function quoteRates($delivery_address, $items, $price, $weight, $weight_unit = 'g') // Default weight unit to gram
    {
        $data = [
            'weight' => $weight,
            'price' => $price,
            'items' => $items,
            'recipient' => $delivery_address
        ];

 //       if (get_woocommerce_currency() && get_woocommerce_currency() !== '' && class_exists('WCML_Multi_Currency') && get_option('smartpack_filter_basket_by_currency') === 'yes') {
 //           $data['data']['attributes']['filter_by_currency'] = get_woocommerce_currency();
 //       }

        $key = 'smartpack:' . md5($this->accessToken . json_encode($data));

        if ($cache = get_transient($key)) {
            return $cache;
        }

        try {
            $res = $this->request($this->endpoint . '/shopcheckout/list', 'post', $data);
            $body = $res['data'];
            // Cache it for up to a day
            set_transient($key, $body, 60 * 60 * 24);
        } catch (Exception $e) {
            error_log('Rate quote failed: ' . $e->getMessage());
            $body = null;
        }


        return $body;
    }


    /**
     * Find drop-doints close to the entered
     * address
     *
     * @param int $delivery_method
     * @param string $address
     * @param string|int $zip
     * @param string $city
     * @param string $country_code
     *
     * @return array array of drop-points
     */
    public function searchDropPoint($delivery_method, $address, $zip, $city, $country_code)
    {

        try {
            $res = $this->request($this->endpoint . "/droppoint/list/${delivery_method}?country=${country_code}&zipcode=${zip}&street=${address}", "get", null);
            $body = $res['data'];
        } catch (Exception $e) {
            error_log('Unable to GET drop point: ' . $e->getMessage());
            return [];
        }

        return $body;
    }

    /**
     * Overruling a built in WooCommerce method
     * Get the rate currently selected in checkout
     *
     * @return WC_Shipping_Rate
     */
    public function getSelectedRate()
    {
        if (get_option('smartpack_remove_cart_recalculation') != 'yes') {
          WC()->cart->calculate_totals();
        }

        $chosen_rates = WC()->session->chosen_shipping_methods;

        foreach (WC()->shipping->get_packages() as $package) {
            /** @var WC_Shipping_Rate $rate */
            foreach ($package['rates'] as $rate) {
                if (in_array($rate->get_id(), $chosen_rates)) {
                return $rate->get_id();
                }
            }
        }

        if (is_array($chosen_rates)) {
            return reset($chosen_rates);
        }


        if ($_POST && isset($_POST["shipping_method"])) {
            if (is_array($_POST["shipping_method"])) {
                return reset($_POST["shipping_method"]);
            }
            if (is_string($_POST["shipping_method"])) {
                return $_POST["shipping_method"];
            }
        }

        return null;
    }


    protected function request($endpoint, $method, $data = null)
    {
        $headers = [
            'X-SmartPack-AppId' =>  $this->appId,
            'X-SmartPack-AccessToken' => $this->accessToken,
            'Content-Type' => 'application/vnd.api+json',
            'Accept' => 'application/vnd.api+json'
        ];

        $response = null;

        try {
            if ($method == 'post') {
                $payload = json_encode($data);

                $response = wp_remote_retrieve_body(wp_remote_post($endpoint, [
                  'body' => $payload,
                  'headers' => $headers
                ]));

            } else {
              $response = wp_remote_retrieve_body(wp_remote_get($endpoint, [ 'headers' => $headers ]));
            }
        } catch (Exception $e) {
          error_log("Failed to make request to Smartpack api: " . $e->getMessage());
          return false;
        }

        $json = json_decode($response, true);

        return $json;
    }

    /**
     * Hacky hack.
     * echo <tr></tr> for choosing a drop-point
     * caught in WooCommerces ob_start
     * and returned in the response of the request
     *
     * @return void
     */
    public function printDropPointSelector()
    {
        $rate = $this->getSelectedRate();
        $require_dp = false;
        $method_key = false;

        if ($rate) {
          if (is_string($rate)) {
            if (preg_match("/smartpack/", $rate)) {
              $method_arr = explode("|", $rate);
              if ($method_arr[0] == '1') {
                $require_dp = true;
                $method_key = end($method_arr);
              }
            }
          } else {
            if ($rate->get_meta_data() && array_key_exists('drop_point', $rate->get_meta_data()) && $rate->get_meta_data()['drop_point']) {
              $require_dp = true;
            }
          }
        }
        //echo '<script>console.log("Debug: ' . $rate . '");</script>';

        if (!$require_dp) {
            // Rate does't require a drop-point, so we just display an empty table-header
            echo "<tr><td colspan='2'>";
        } else {
            // Rate requires a drop-point

            // We display a gorgeous drop-point selection map
            // if the customer have a google maps api key
            if (strlen(get_option('smartpack_google_maps_api_key', '')) > 1) {
                $this->modalWithMap($rate);
            } else {
                $this->genericRadioButtons($rate, $method_key);
            }
        }
    }

    protected function modalWithMap($rate)
    {
      wc_get_template( '/templates/drop_point_modal.php', array(), '', plugin_dir_path( __FILE__ ));
    }

    protected function genericRadioButtons($rate, $method_key = false)
    {
        // Display a boring table for selecting the drop-point
        $result = array();

        // Default state; No drop-point selected
        $drop_points = [];

        // Grab delivery_method from meta_data if, rate is not set
        $method_key = $method_key ? $method_key : $rate->get_meta_data()['delivery_method'];

        // Find an appropriate address
        $address = WC()->checkout->get_value('shipping_address_1') ? WC()->checkout->get_value('shipping_address_1') : WC()->checkout->get_value('billing_address_1');
        $zip = WC()->checkout->get_value('shipping_postcode') ? WC()->checkout->get_value('shipping_postcode') : WC()->checkout->get_value('billing_postcode');
        $city =  WC()->checkout->get_value('shipping_city') ? WC()->checkout->get_value('shipping_city') : WC()->checkout->get_value('billing_city');
        $country_code = WC()->checkout->get_value('shipping_country') ? WC()->checkout->get_value('shipping_country') : WC()->checkout->get_value('billing_country');

        // Don't search droppints when there is no zip code
        if (!isset($zip) || (isset($zip) && ($zip == '0' || strlen($zip) < 1))){
            return;
        }

        // Find all available drop points
        // and format them prettily
        // Ignore errors and just continue
        // with the next DP
        try {
            $result = $this->searchDropPoint($method_key, $address, $zip, $city, $country_code);
            foreach ($result as $dp) {
                $key = urlencode(json_encode(['id' => $dp['id'], 'street1' => $dp['address']['street1'], 'zipcode' => $dp['address']['zipcode'], 'city' => $dp['address']['city'], 'country' => $dp['address']['country'], 'name' => $dp['name']]));
                $drop_points[$key] = "{$dp['name']}, {$dp['address']['street1']} {$dp['address']['zipcode']} {$dp['address']['city']}";
            }
        } catch (Exception $e) {
            error_log($e->getMessage());
            $drop_points = ['null' => ___("No pickup point found")];
        }

        // Start output buffering
        // (Allows us to capture echo statements)
        ob_start();

        echo "<tr><td colspan='2'>";
        if (count($drop_points) <= 0) {
          echo ___("Unable to find droppoint. Please check your address");
        } else {
          woocommerce_form_field('sp_drop_point_blob', [
            'type' => 'select',
            'label' => null,
            'required' => false,
            'options' => $drop_points
          ], '0');
        }

        echo "</td></tr>";

        // Stop this level of output buffering
        // and get the clean HTML
        $html = ob_get_clean();

        // Pass the full html to WooCommerces output buffering
        echo apply_filters('smartpack_drop_point_selector_html', $html, $drop_points, $result);
    }
}
