<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class SmartpackShippingRates
 *
 *
 * See https://docs.woocommerce.com/document/shipping-method-api/#section-5
 */
class SmartpackShippingRates extends WC_Shipping_Method
{

    public static $quoteCounter = 0;

    /**
     * SmartpackShippingRates constructor.
     */
    public function __construct()
    {
        $this->id = 'smartpack';
        $this->method_title = "Smartpack Checkout";
        $this->method_description = "Setup you checkout flow in your 'smartpack customer > settings > checkout' page!";
        $this->title = "Smartpack Checkout";
        $this->enabled = "yes";

        // Dont show tab in settings
        $this->supports = [];

        $this->init();
    }

    /**
     * Required to be here by Woo
     *
     * @return void
     */
    protected function init()
    {
      //
    }


    /**
     * Framework method. Must call $this->add_rate($rate) to add rates
     *
     * @param array $package
     */
    public function calculate_shipping($package = [])
    {
        self::$quoteCounter++;

        if (self::$quoteCounter >= 20) {
          return;
        }

        try {
            $api = SmartpackAPI::instance();
        } catch (Exception $e) {
            error_log('Smartpack plugin not configured');
            return;
        }

        $total = 0;
        $coupon_free_shipping = false;
        $items = [];
        $weight_uom = get_option('woocommerce_weight_unit');
        $weight_multiplier = $weight_uom == 'kg' ? 1000 : 1;
        $weight_total = 0;

        // Calculate cart total incl. taxes
        if (count($package["contents"]) > 0) {
            foreach ($package["contents"] as $content) {
                // Calculate total
                $total += $content["line_total"] + $content["line_tax"];

                // Get apropriate product info for the quote
                if ((int) $content["variation_id"] > 0) {
                  try {
                    $product = new WC_Product_Variation($content["variation_id"]); // Variation inherits Product
                  } catch (Exception $e) { // They might have removed the variation.
                    $product = new WC_Product($content["product_id"]);
                  }
                } else {
                  $product = new WC_Product($content["product_id"]);
                }
                // Get apropriate weight
                $weight = (double) $product->get_weight() * $weight_multiplier;
                $weight_total += ($weight * (double) $content["quantity"]);
                $virtual = $product->get_virtual();
                $pid = $product->get_id();
                $sku = $product->get_sku() ? $product->get_sku() : ('ext_id' . $pid);


                if ((float) $content["line_subtotal"] > 0 && (int) $content["quantity"] > 0) {
                  $price = (float) $content["line_subtotal"] / (float) $content["quantity"];
                } else {
                  $price = 0;
                }


                $items[] = [
                  'quantity' => $content['quantity'],
                  'sku' => $sku,
                  'price' => $price,
                  'weight' => $weight,
                  'requires_shipping' => !$virtual
                ];
            }
        }

        // Check if any coupon codes are applied
        if (count($package['applied_coupons']) > 0) {
            foreach ($package['applied_coupons'] as $coupon) {
                $obj = new WC_Coupon($coupon);
                //$total = $total - $obj->amount;

                // check if coupon grants free shipping
                if ($obj->get_free_shipping($coupon) == true) {
                  $coupon_free_shipping = true;
                }
            }
        }

        $delivery_address = array(
            'street1' => $package['destination']['address'],
            'street2' => $package['destination']['address_2'],
            'city' => $package['destination']['city'],
            'zipcode' => $package['destination']['postcode'],
            'country' => $package['destination']['country']
        );
        

        if (isset($_POST) && isset($_POST['post_data'])) {
          wp_parse_str(sanitize_text_field($_POST['post_data']), $postData);
  
          if (isset($postData)) {
            $companyName = null;
            $shipping_company = sanitize_text_field($postData['shipping_company']);
            $billing_company = sanitize_text_field($postData['billing_company']);
  
            if (isset($shipping_company) && strlen($shipping_company) > 0) {
              $companyName = $shipping_company;
            } elseif (isset($billing_company) && strlen($billing_company) > 0) {
              $companyName = $billing_company;
            }
  
            if ($companyName) {
                $delivery_address['name'] = $companyName;
            }

            $phone = sanitize_text_field($postData['billing_phone']);
            $email = sanitize_text_field($postData['billing_email']);

            if ($phone) {
                $delivery_address['phone'] = $phone;
            }

            if ($email) {
                $delivery_address['email'] = $email;
            }
          }
        }



        $test_fallback = get_option('smartpack_test_fallback_methods') == 'yes';
        $res = $api->quoteRates($delivery_address, $items, $total, $weight_total);

        if (!$test_fallback && !is_null($res)) {
            foreach ($res as $rate) {
                $method_key = $this->id . '|' . $rate['code'];

                if ($rate['isDropPoint']) {
                  $method_key = '1|' . $method_key;
                };

                $rate_params = [
                    'id' => $method_key,
                    'label' => $rate['name'],
                    'cost' => ($coupon_free_shipping ? 0 : $rate['price']),
                    'calc_tax' => 'per_order',
                    'taxes' => '',
                    'meta_data' => [
                        'drop_point' => $rate['isDropPoint'],
                        'delivery_method' => $rate['code'],
                    ]
                ];

    //            if (class_exists('WCML_Multi_Currency') && get_option('smartpack_filter_basket_by_currency') === 'yes') {
    //              $wcml_options = get_option('_wcml_settings');
    //              if ($wcml_options['enable_multi_currency']) {
    //                $rate_params['cost'] = $this->prepare_for_exchange_rates($rate, $rate_params);
    //              }
    //            }

                $this->add_rate($rate_params);
            }
        }
        else {
            $fallback = get_option('smartpack_fallback_methods');
            foreach (explode(",", $fallback) as $fb_data_str) {
                $fb_data = explode("|", $fb_data_str);
                $free_shipping = $coupon_free_shipping || $total > (double)$fb_data[3];
                $rate_params = [
                    'id' => $fb_data[0],
                    'label' => $fb_data[1],
                    'cost' => ($free_shipping ? 0 : $fb_data[2]),
                    'calc_tax' => 'per_order',
                    'taxes' => '',
                    'meta_data' => [
                        'delivery_method' => $fb_data[0]
                    ]
                ];
                $this->add_rate($rate_params);
            }
        }
    }

    /**
     * Prepare value  for an exchange between two currencies
     * Only used if WCML is in use
     *
     * @param array $rate
     * @param array $params
     * @return void
     */
    public function prepare_for_exchange_rates($rate, $params)
    {
      if ($params['cost'] == 0) {
        return 0;
      }

      if ($rate['currency'] == get_option('woocommerce_currency')) {
        return $params['cost'];
      }

      // Unfuck the forced currency conversion even though we return the requested currency
      $multiCurrency = new WCML_Multi_Currency;
      return $multiCurrency->prices->unconvert_price_amount($rate['price'], $rate['currency']);
    }
}
