<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 *
 * Plugin Name: Smartpack Checkout
 * Description: Checkout for WooCommerce
 * Author: Smartpack
 * Author URI: https://www.smartpack.dk
 * Version: 1.0.0
 */
if (!function_exists('is_plugin_active_for_network')) {
    require_once(ABSPATH . '/wp-admin/includes/plugin.php');
}

if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins'))) || is_plugin_active_for_network('woocommerce/woocommerce.php')) {
    // Loads dependencies e.g. Guzzle
    require 'vendor/autoload.php';


    // REST API Adapter
    require_once('SmartpackAPI.php');
    require_once('SmartpackHelpers.php');
    require_once('SmartpackOrderHtml.php');

    /**
     * Wait for WC_Shipping_Method class to be loaded
     */
    add_action('woocommerce_shipping_init', function () {
        if (class_exists('SmartpackShippingRates')) {
            return;
        }

        require_once 'SmartpackShippingRates.php';
    });


    /**
     * Initialise shipping rates
     */
    add_filter('woocommerce_shipping_methods', function ($methods) {
        $methods['smartpack'] = 'SmartpackShippingRates';
        return $methods;
    });


    /**
     * Hook into "Your order" summary
     *
     * Add a row for drop point selector of chosen shipping rate requires it
     */
    add_action('woocommerce_after_shipping_rate', 'smartpack_drop_point_selector_location', 20, 2);

    function smartpack_drop_point_selector_location($shipping_rate, $index)
    {
        static $rate_count = 0;
        static $total_rates;
    
        if (!WC()->cart->needs_shipping() || !WC()->cart->show_shipping()) {
            return;
        }

        if (is_null($total_rates)) {
            $total_rates = count(WC()->session->get('shipping_for_package_0')['rates']);
        }
    
    
        $rate_count++;
    
        if ($rate_count === $total_rates) { // Only output drop point selector after the last shipping rate
            try {
                SmartpackAPI::instance()->printDropPointSelector();
            } catch (Exception $e) {
                echo "<em>Smartpack plugin activated but not configured. Configure it now under WooCommerce > Settings > Shipping > Shipping options</em>";
                return;
            }
        }
    }


    /**
     *  Validate "sp_drop_point_blob" POST field on checkout
     */
    add_action('woocommerce_checkout_process', function () {
        if (!WC()->cart->needs_shipping()) {
            return;
        }

        try {
            $api = SmartpackAPI::instance();
        } catch (Exception $e) {
            error_log($e->getMessage());
            return;
        }

        $rate = $api->getSelectedRate();

        if (!$rate) {
            wc_add_notice(___('No shipping rate chosen'), 'error');
            return;
        }

        if (is_string($rate)) {
            if (preg_match( "/smartpack/", $rate)) {
                $method_arr = explode("|", $rate);
                if ($method_arr[0] != '1') {
                    return;
                }
            }
        } else {
            if ($rate->get_meta_data() && (!array_key_exists('requires_drop_point', $rate->get_meta_data()) || !$rate->get_meta_data()['requires_drop_point'])) {
                return;
            }
        }

        if (strlen(get_option('smartpack_google_maps_api_key', '')) > 1) {
            // Validate that the POST attribute is set
            if (!isset($_POST['sp_drop_point_blob'])) {
                $translation = ___('please choose a drop point');
                wc_add_notice($translation, 'error');

                return;
            }

            # stripslashes remove all backslashes that cannot properly be decoded
            $sp_drop_point_blob = json_decode(sanitize_text_field(urldecode(stripslashes($_POST['sp_drop_point_blob']))), true);

            // Validate again, post-parse
            if (!$sp_drop_point_blob) {
                $translation = ___('please choose a drop point');
                wc_add_notice($translation, 'error');
                return;
            }
        }
    });


    /**
     * Set drop point as delivery address
     * if rate needs drop-point
     *
     */
    add_action('woocommerce_checkout_update_order_meta', function ($order_id) {
        $order = new WC_Order($order_id);
        $orderData = $order->get_data();

        if (!isset($_POST['sp_drop_point_blob'])) {
            return;
        }

        # stripslashes remove all backslashes that cannot properly be decoded
        $sp_drop_point_blob = json_decode(sanitize_text_field(urldecode(stripslashes($_POST['sp_drop_point_blob']))), true);


        // Use customer name, but drop point address
        update_post_meta($order_id, '_drop_point_id', sanitize_text_field($sp_drop_point_blob['id']));
        update_post_meta($order_id, '_drop_point_name', sanitize_text_field($sp_drop_point_blob['name']));
        update_post_meta($order_id, '_drop_point_street1', sanitize_text_field($sp_drop_point_blob['street1']));
        update_post_meta($order_id, '_drop_point_city', sanitize_text_field($sp_drop_point_blob['city']));
        update_post_meta($order_id, '_drop_point_zipcode', sanitize_text_field($sp_drop_point_blob['zipcode']));
        update_post_meta($order_id, '_drop_point_country', sanitize_text_field($sp_drop_point_blob['country']));
    });



    /**
     * Allow the user to change the shipping method
     * directly from the order page
     */
    add_action('woocommerce_admin_order_data_after_order_details', 'show_on_order');
    function show_on_order()
    {
        try {
           
            $wooOrder = new WC_Order($_GET["post"]);
            if (isset($_GET["smartpack_change_droppoint"]) && $_GET["smartpack_change_droppoint"] == 'true') {
                change_droppoint(
                    $wooOrder,
                    sanitize_text_field($_GET["dp_id"]),
                    sanitize_text_field(urldecode($_GET["dp_street"])),
                    sanitize_text_field($_GET["dp_zip"]),
                    sanitize_text_field(urldecode($_GET["dp_city"])),
                    sanitize_text_field(urldecode($_GET["dp_name"])),
                    sanitize_text_field($_GET["dp_country"])
                );
            }

            if (isset($_GET["smartpack_change_shipping_method"]) && $_GET["smartpack_change_shipping_method"] == 'true') {
                change_shipping_method(
                    $wooOrder,
                    sanitize_text_field($_GET["sp_method_key"]),
                    sanitize_text_field(urldecode($_GET["name"]))
                );
            }

            if (isset($_GET['smartpack_change_shipping_method']) || isset($_GET['smartpack_change_droppoint'])) {
                unset($_GET['smartpack_change_shipping_method']);
                unset($_GET['smartpack_change_droppoint']);
                unset($_GET['dp_id']);
                unset($_GET['dp_street']);
                unset($_GET['dp_zip']);
                unset($_GET['dp_city']);
                unset($_GET['dp_name']);
                unset($_GET['dp_country']);
                unset($_GET['sp_method_key']);
                unset($_GET['name']);
                $qs = http_build_query($_GET);

                $base = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . explode('?', $_SERVER['REQUEST_URI'], 2)[0];
                $base .= '?' . $qs;

                header('Location: ' . $base);
            }

            $order_html = new SmartpackOrderHtml($wooOrder);
            $order_html->render_html();

        } catch (Exception $e) {
            error_log($e->message);
            return;
        }
    }

    /**
     * Change the shipping method for an order
     *
     * @param WC_Order $woo_order
     * @param string $method_key
     * @param string $method_name
     * @return void
     */
    function change_shipping_method($woo_order, $method_key, $method_name)
    {
        if (method_exists($woo_order, 'get_shipping_method') && $woo_order->get_shipping_method()) {
            foreach ($woo_order->get_shipping_methods() as $shipping_method) {
                $shipping_method->set_method_title($method_name);
                $shipping_method->update_meta_data('delivery_method', $method_key);
                $woo_order->save();
            }
        } else {
            $shipping_method = new WC_Shipping_Rate($method_key, $method_name, 0, array());
            $shipping_method->add_meta_data('delivery_method', $method_key);
            $woo_order->add_shipping($shipping_method);
        }
    }

    /**
     * Change the drop-point to deliver to
     * for an order
     *
     * @param WC_Order $woo_order
     * @param int $dp_id
     * @param string $dp_street
     * @param string $dp_zip
     * @param string $dp_city
     * @param string $dp_name
     * @param string $dp_country
     * @return void
     */
    function change_droppoint($woo_order, $dp_id, $dp_street, $dp_zip, $dp_city, $dp_name, $dp_country)
    {
        update_post_meta($woo_order->id, '_drop_point_id', $dp_id);
        update_post_meta($woo_order->id, '_drop_point_name', $dp_name);
        update_post_meta($woo_order->id, '_drop_point_street1', $dp_street);
        update_post_meta($woo_order->id, '_drop_point_city', $dp_city);
        update_post_meta($woo_order->id, '_drop_point_zipcode', $dp_zip);
        update_post_meta($woo_order->id, '_drop_point_country', $dp_country);
    }


    /**
     * Adds Smartpack configuration settings to WooCommerce > Settings > Shipping > Shipping options
     */
    add_filter('woocommerce_get_settings_shipping', function ($settings, $current_section) {
        $settings[] = [
            'name' => 'Smartpack Settings',
            'type' => 'title',
            'desc' => 'Get your connect token from the woocommerce app in smartpack, configure your checkout in smartpack (customer > settings > shop checkout)',
            'id' => 'smartpack'
        ];

        $settings[] = [
            'name' => 'Smartpack Connect Token',
            'type' => 'text',
            'id' => 'smartpack_access_str'
        ];

        $settings[] = [
            'name' => 'Google Maps API Key',
            'default' => '',
            'type' => 'text',
            'id' => 'smartpack_google_maps_api_key'
        ];

        $settings[] = [
          'name' => 'Stop cart recalculations during rate quote. Try enabling this, if you experience issues with coupons',
          'default' => 'no',
          'type' => 'checkbox',
          'id' => 'smartpack_remove_cart_recalculation'
        ];

        $settings[] = [
            'name' => 'Delivery Method Fallback (if service unavailable)',
            'type' => 'text',
            'desc' => 'method_key|method_name|price|free_shipping_limit,method_key|method_name|price|free_shipping_limit,...',
            'id' => 'smartpack_fallback_methods'
        ];

        
        $settings[] = [
            'name' => 'Test Delivery Method Fallback',
            'default' => 'no',
            'type' => 'checkbox',
            'id' => 'smartpack_test_fallback_methods'
        ];


        $settings[] = [
            'type' => 'sectionend',
            'id' => 'smartpack'
        ];

        return $settings;
    }, 10, 2);
    

        /**
     * add droppoint to shipping address
     */
    add_filter('woocommerce_order_get_formatted_shipping_address', function ($address, $raw_address, $order) {
		$drop_point_id = $order->get_meta('_drop_point_id');

		if(!empty($drop_point_id)) {
			if(!empty($address)) {
				$address .= '<br><br>';
			}
			$address .= '<b>Udleveringssted:</b><br/>' .  $order->get_meta('_drop_point_name') . '<br/>' .   $order->get_meta('_drop_point_street1') . '<br/>' .  $order->get_meta('_drop_point_zipcode') . ' ' .  $order->get_meta('_drop_point_city');
		}
		return $address;
    }, 10, 3);


    // Load Google Maps dependencies only if
    // the user is using out Google Maps map
    add_action('wp_enqueue_scripts', 'enqueue_frontend');

    // Enqueue Backend scripts
    add_action('admin_enqueue_scripts', 'enqueue_backend');

    /**
     * Enqueue frontend scripts
     * Only used for displaying the incredible
     * drop-point selector
     *
     * @return void
     */
    function register_frontend()
    {
        wp_register_style("smartpack_css", plugins_url("css/smartpack.min.css", __FILE__), array(), SmartpackAPI::VERSION, false);

        if (get_option('smartpack_google_maps_api_key', false)) {
            wp_register_script("smartpack_maps", "https://maps.googleapis.com/maps/api/js?key=" . get_option('smartpack_google_maps_api_key'), array(), SmartpackAPI::VERSION, false);
        }

        wp_register_script("smartpack_drop_point", plugins_url("js/drop_point.js", __FILE__), array('jquery'), SmartpackAPI::VERSION, false);
    }

    // Add the frontend scripts to the dom
    function enqueue_frontend()
    {
        // Only load assets if woocommerce
        if (!function_exists('is_woocommerce')) {
            return;
        }

        // Only load assets if either checkout or cart
        if (!is_cart() && !is_checkout()) {
            return;
        }

        // CSS
        wp_register_style("smartpack_css", plugins_url("css/smartpack.min.css", __FILE__), array(), SmartpackAPI::VERSION, false);
        wp_enqueue_style('smartpack_css');

        // JS
        if (get_option('smartpack_google_maps_api_key', false)) {
            wp_register_script("smartpack_maps", "https://maps.googleapis.com/maps/api/js?key=" . get_option('smartpack_google_maps_api_key'), array(), SmartpackAPI::VERSION, false);
            wp_enqueue_script('smartpack_maps');
        }

        wp_register_script("smartpack_drop_point", plugins_url("js/drop_point.js", __FILE__), array('jquery'), SmartpackAPI::VERSION, false);
        wp_enqueue_script("smartpack_drop_point");

        $smartpack_nonce = wp_create_nonce("smartpack_nonce");
        wp_localize_script("smartpack_drop_point", "smartpack_ajax_object", array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'smartpack_nonce' => $smartpack_nonce
        ));
    }

    // Add backend scripts to the dom
    function enqueue_backend()
    {
        wp_register_script("smartpack_backend", plugins_url("js/backend.min.js", __FILE__), array('jquery'), SmartpackAPI::VERSION, true);
        wp_enqueue_script('smartpack_backend');
    }

    // Bind ajax calls to our methods
    add_action('wp_ajax_get_shops', 'locate_drop_points');
    add_action('wp_ajax_nopriv_get_shops', 'locate_drop_points');

    /**
     * Locate drop_points for the shipping_rate
     *
     * @return void
     */
    function locate_drop_points()
    {
        // Smartpack API Instance
        $api = SmartpackAPI::instance();
        $rate = $api->getSelectedRate();
        $require_dp = false;
        $method_key = null;
        if ($rate) {
            if (is_string($rate) && preg_match("/smartpack/", $rate)) {
                $method_arr = explode("|", $rate);
                if ($method_arr[0] == '1') {
                    $require_dp = true;
                    $method_key = end($method_arr);
                }
            } else {
                if ($rate->get_meta_data() && $rate->get_meta_data()['requires_drop_point'] && $rate->get_meta_data()['delivery_method']) {
                    $require_dp = true;
                    $method_key = $rate->get_meta_data()['delivery_method'];
                }
            }
        }

        if ($require_dp) {
            try {
                $country_code = WC()->checkout->get_value('shipping_country');

                $result = $api->searchDropPoint($method_key, sanitize_text_field($_POST['address']), sanitize_text_field($_POST['zip']), null, $country_code);

                wp_send_json_success(['drop_points' => $result]);
            } catch (Exception $e) {
                error_log($e->getMessage());
            }
        }
    };
}
